/* Open.Net Authentication Daemon
 * A simple test client
 *
 * Copyright (C) 2001 Martin Hedenfalk <mhe@it.kth.se>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* $Id: client.c,v 1.12 2002/04/19 11:51:15 mhe Exp $
 */

#include <stdio.h>
#include <ctype.h>
#include <unistd.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <stdlib.h>
#include <time.h>

FILE *rfp = 0, *wfp = 0;

void usage(void)
{
  printf("usage: client login USER DOMAIN PASS IP MAC\n"
		 "       client logout USER DOMAIN IP MAC\n"
		 "       client stats\n"
		 "       client list\n"
		 "       client list DOMAIN\n"
		 "       client list USER DOMAIN\n");
  exit(1);
}


/* trims leading and trailing whitespace from STR
 */
char *trim(char *str)
{
	char *e;

	while(isspace(*str))
		++str;
	e = str + strlen(str);
	while(--e != str && isspace(*e))
		/* do nothing */ ;
	*(e+1) = 0;

	return str;
}

static char *getline(void)
{
	static char buf[1024];

	if(fgets(buf, 1024, rfp) == 0)
		return 0;
	return trim(buf);
}

static void stats(void)
{
	char *statline;
	unsigned int nloggedin;
	unsigned long int nlogins, nfailauth, nfailreq, uptime;
	unsigned int d, h, m, s;

	printf("Sending STATS request... ");
	fflush(stdout);
	fprintf(wfp, "STATS\r\n");
	fflush(wfp);
	printf("%s\n", getline());
	statline = getline();
	sscanf(statline, "%u %lu %lu %lu %lu",
		   &nloggedin, &nlogins, &nfailauth, &nfailreq, &uptime);
	printf("number of users currently logged in: %u\n", nloggedin);
	printf("total number of logins: %lu\n", nlogins);
	printf("failed authentications: %lu\n", nfailauth);
	printf("failed requests: %lu\n", nfailreq);
	d = uptime / (60*60*24);
	uptime = uptime - d*24*60*60;
	h = uptime / (60*60);
	uptime = uptime - h*60*60;
	m = uptime / 60;
	uptime = uptime - m*60;
	s = uptime;
	printf("uptime: %u days, %u hours, %u minutes, %u seconds\n", d, h, m, s);
}

static void login(char *user, char *domain, char *pass, char *ip, char *mac)
{
	printf("Sending LOGIN request... ");
	fflush(stdout);
	fprintf(wfp, "LOGIN\rUSER=%s\rDOMAIN=%s\rPASS=%s\rIP=%s\rMAC=%s\r\n",
			user, domain, pass, ip, mac);
	fflush(wfp);
	printf("%s\n", getline());
}

static void logout(char *user, char *domain, char *ip, char *mac)
{
	printf("Sending LOGOUT request... ");
	fflush(stdout);
	fprintf(wfp, "LOGOUT\rUSER=%s\rDOMAIN=%s\rIP=%s\rMAC=%s\r\n",
			user, domain, ip, mac);
	fflush(wfp);
	printf("%s\n", getline());
}

static void list(char *user, char *domain)
{
	unsigned int n = 0;
	char *l;
	char *status, *username, *domainname, *ip, *mac, *logintime;
	time_t lt;

	printf("Sending LIST request... ");
	fflush(stdout);
	if(user && domain)
		fprintf(wfp, "LIST\rUSER=%s\rDOMAIN=%s\r\n", user, domain);
	else if(domain)
		fprintf(wfp, "LIST\rDOMAIN=%s\r\n", domain);
	else
		fprintf(wfp, "LIST\r\n");
	fflush(wfp);
	printf("%s\n", getline());

	while((l = getline()) != 0) {
		sscanf(l, "%as %as %as %as %as %as",
			   &status, &username, &domainname, &ip, &mac, &logintime);
		lt = atol(logintime);
		printf("%s %s %s %s %s %s", 
			   status, username, domainname, ip, mac, ctime(&lt));
		free(status);
		free(username);
		free(domainname);
		free(ip);
		free(mac);
		free(logintime);
		++n;
	}
	printf("-----------------------\n");
	printf("Totally %u users logged in\n", n);
}

int main(int argc, char **argv)
{
	int sockfd;
	struct sockaddr_un saddr;
	char *sockfile = "/tmp/oasis";

	if(argc == 1 || argc > 7)
		usage();

	sockfd = socket(PF_UNIX, SOCK_STREAM, 0);
	if(sockfd == -1) {
		perror("socket");
		return 2;
	}

	saddr.sun_family = AF_UNIX;
	strcpy(saddr.sun_path, sockfile);

	printf("connecting to %s... ", sockfile);
	fflush(stdout);
	if(connect(sockfd, (struct sockaddr *)&saddr, sizeof(saddr)) != 0) {
		perror("connect");
		return 3;
	}
	printf("connected\n");

	rfp = fdopen(sockfd, "r");
	if(rfp == 0) {
		printf("fdopen(r) failed\n");
		return 5;
	}
	wfp = fdopen(sockfd, "w");
	if(wfp == 0) {
		printf("fdopen(w) failed\n");
		return 6;
	}

	if(strcmp(argv[1], "login") == 0) {
		if(argc != 7)
			usage();
		login(argv[2], argv[3], argv[4], argv[5], argv[6]);
	} else if(strcmp(argv[1], "logout") == 0) {
		if(argc != 6)
			usage();
		logout(argv[2], argv[3], argv[4], argv[5]);
	} else if(strcmp(argv[1], "stats") == 0) {
		if(argc != 2)
			usage();
		stats();
	} else if(strcmp(argv[1], "list") == 0) {
		if(argc == 2)
			list(0, 0);
		else if(argc == 3)
			list(0, argv[2]);
		else if(argc == 4)
			list(argv[2], argv[3]);
		else
			usage();
	}

	close(sockfd);
	return 0;
}
