/*
 *
 *
 * Written by Martin Hedenfalk <mhe@it.kth.se>
 *
 * Copyright (c) 2002, Royal Institute of Technology, Stockholm, Sweden
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. Neither the name of the Institute nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/* $Id: fwssl.h,v 1.8 2002/04/16 15:49:29 mhe Exp $
 */

#ifndef _fwssl_h_
#define _fwssl_h_

#include "common.h"

#include <unistd.h>
#include <string.h>
#include <netdb.h>
#include <openssl/ssl.h>
#include <openssl/x509.h>
#include <openssl/pem.h>
#include <openssl/err.h>
#include <netinet/in.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>

#define FWCMD_OPEN 1
#define FWCMD_CLOSE 2
#define FWCMD_RESET 3
#define FWCMD_SYNC_USER 4
#define FWCMD_STATS 5
/*#define FWCMD_SYNC_DOMAIN 5*/

struct fwreq_t {
	int cmd;
	struct in_addr ip;
	unsigned char mac[6];
};
typedef struct fwreq_t fwreq_t;

struct fwreply_t {
	req_status_t status;
	int nstats;
};
typedef struct fwreply_t fwreply_t;

/* struct fwstat_t { */
/* 	struct in_addr ip; */
/* 	unsigned char mac[6]; */
/* }; */
/* typedef struct fwstat_t fwstat_t; */

void logmsg_ssl(int fac);

SSL_CTX *init_ctx(const char *certfile,
				  const char *keyfile,
				  const char *valid_cert_file,
				  int client);
int init_ssl_daemon(SSL_CTX *ctx, int port, int (*callback)(SSL *));
SSL *init_ssl_client(SSL_CTX *ctx, const char *host, int port);
int check_ssl_io(int ret, SSL *ssl, int expected);
req_status_t fw_open_local(char *firewall,
						   struct in_addr *ip,
						   unsigned char mac[6]);
req_status_t fw_close_local(char *firewall,
							struct in_addr *ip,
							unsigned char mac[6]);
req_status_t fw_reset_local(char *firewall);

#endif
