/* $Id: list.h,v 1.5 2002/02/20 20:51:48 mhe Exp $
 *
 * list.h -- generic linked list
 *
 * Yet Another FTP Client
 * Copyright (C) 1998-2001, Martin Hedenfalk <mhe@stacken.kth.se>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version. See the GPL for more details.
 */

#ifndef _list_h_
#define _list_h_

struct listitem_t {
	void *data;
	struct listitem_t *next, *prev;
};
typedef struct listitem_t listitem_t;

typedef void (*list_delfunc_t)(void *);
typedef int (*list_cmpfunc_t)(const void *, const void *);

struct list_t {
	list_delfunc_t delfunc;
	int size;
	struct listitem_t *first, *last;
};
typedef struct list_t list_t;

list_t *list_new(list_delfunc_t del_func);
listitem_t *list_insert(list_t *list, void *data);
void list_clear(list_t *list); /* remove all items in lp */
listitem_t *list_detach(list_t *lp, listitem_t *li);
void list_remove(list_t *list, listitem_t *item);
void list_destroy(list_t *list);
listitem_t *list_search(list_t *lp, list_cmpfunc_t cmpfunc,
						const void *arg);
int list_size(list_t *list);

void list_movetofront(list_t *lp, listitem_t *li);
void list_movetoback(list_t *lp, listitem_t *li);

#endif
